@@ -5,8 +5,10 @@ from ixnetwork_restpy import TestPlatform, SessionAssistant
 from abstract_open_traffic_generator.api import *
 from abstract_open_traffic_generator.config import *
 from abstract_open_traffic_generator.control import *
+from abstract_open_traffic_generator.result import RequestDetail
 from ixnetwork_open_traffic_generator.validation import Validation
 from ixnetwork_open_traffic_generator.vport import Vport
+from ixnetwork_open_traffic_generator.lag import Lag
 from ixnetwork_open_traffic_generator.ngpf import Ngpf
 from ixnetwork_open_traffic_generator.trafficitem import TrafficItem
 from ixnetwork_open_traffic_generator.timer import Timer
@@ -48,8 +50,10 @@ class IxNetworkApi(Api):
         self._running_config = None
         self._config = None
         self._assistant = None
+        self._ixn_errors = list()
         self.validation = Validation(self)
         self.vport = Vport(self)
+        self.lag = Lag(self)
         self.ngpf = Ngpf(self)
         self.traffic_item = TrafficItem(self)
 
@@ -110,7 +114,28 @@ class IxNetworkApi(Api):
             self._set_flow_transmit_state(state.flow_transmit_state)
         elif state.choice == 'port_capture_state':
             self._set_port_capture_state(state.port_capture_state)
+        return self._request_detail()
 
+    def _request_detail(self):
+        request_detail = RequestDetail()
+        errors = self._errors
+        warnings = list()
+        app_errors = self._globals.AppErrors.find()
+        if len(app_errors) > 0:
+            current_errors = app_errors[0].Error.find()
+            if len(current_errors) > 0:
+                for error in current_errors:
+                    match = [o for o in self._ixn_errors if o.Name == error.Name
+                                                    and o.LastModified == error.LastModified]
+                    if len(match) == 0:
+                        if error.ErrorLevel == 'kWarning':
+                            warnings.append("IxNet - {0}".format(error.Description))
+                        if error.ErrorLevel == 'kError':
+                            errors.append("IxNet - {0}".format(error.Description))
+        request_detail.errors = errors
+        request_detail.warnings = warnings
+        return request_detail
+    
     def _set_config_state(self, config_state):
         """Set or update the configuration
         """
@@ -126,6 +151,7 @@ class IxNetworkApi(Api):
             self._ixnetwork.NewConfig()
         else:
             self.vport.config()
+            self.lag.config()
             with Timer(self, 'Devices configuration'):
                 self.ngpf.config()
             with Timer(self, 'Flows configuration'):
@@ -261,7 +287,7 @@ class IxNetworkApi(Api):
         """Connect to an IxNetwork API Server.
         """
         if self._assistant is None:
-            platform = TestPlatform(self._address, rest_port=self._password)
+            platform = TestPlatform(self._address, rest_port=self._port)
             platform.Authenticate(self._username, self._password)
             url = '%s://%s:%s/ixnetworkweb/api/v1/usersettings/ixnrest' % \
                 (platform.Scheme, platform.Hostname, platform.RestPort)
@@ -277,9 +303,11 @@ class IxNetworkApi(Api):
                                                LogLevel=self._log_level)
             self._ixnetwork = self._assistant.Session.Ixnetwork
             self._vport = self._ixnetwork.Vport
+            self._lag = self._ixnetwork.Lag
             self._topology = self._ixnetwork.Topology
             self._traffic = self._ixnetwork.Traffic
             self._traffic_item = self._ixnetwork.Traffic.TrafficItem
+            self._globals = self._ixnetwork.Globals
             if len(self._license_servers) > 0:
                 self._ixnetwork.Globals.Licensing \
                     .LicensingServers = self._license_servers
@@ -290,6 +318,12 @@ class IxNetworkApi(Api):
                 version = "ixnetwork-open-traffic-generator not installed " \
                     "using pip, unable to determine version"
             self.info(version)
+        self._backup_errors()
+    
+    def _backup_errors(self):
+        app_errors = self._globals.AppErrors.find()
+        if len(app_errors) > 0:
+            self._ixn_errors = app_errors[0].Error.find()
 
     def _request(self, method, url, payload=None):
         connection, url = self._assistant.Session._connection._normalize_url(
@@ -351,6 +385,42 @@ class IxNetworkApi(Api):
     def _get_topology_name(self, port_name):
         return 'Topology %s' % port_name
 
+    def select_card_aggregation(self, location):
+        (hostname, cardid, portid) = location.split(';')
+        payload = {
+            'selects': [{
+                'from':
+                '/availableHardware',
+                'properties': [],
+                'children': [{
+                    'child':
+                    'chassis',
+                    'properties': [],
+                    'filters': [{
+                        'property': 'hostname',
+                        'regex': '^%s$' % hostname
+                    }]
+                }, {
+                    'child':
+                    'card',
+                    'properties': ['*'],
+                    'filters': [{
+                        'property': 'cardId',
+                        'regex': '^%s$' % abs(int(cardid))
+                    }]
+                }, {
+                    'child':
+                    'aggregation',
+                    'properties': ['*'],
+                    'filters': []
+                }],
+                'inlines': []
+            }]
+        }
+        url = '%s/operations/select?xpath=true' % self._ixnetwork.href
+        results = self._ixnetwork._connection._execute(url, payload)
+        return results[0]['chassis'][0]['card'][0]
+
     def select_chassis_card(self, vport):
         pieces = vport['connectionStatus'].split(';')
         payload = {
