@@ -1,3 +1,4 @@
+from time import time
 from ixnetwork_open_traffic_generator.timer import Timer
 from ixnetwork_open_traffic_generator.customfield import CustomField
 
@@ -23,11 +24,11 @@ class TrafficItem(CustomField):
         ('bytes_rx_rate', 'Rx Rate (Bps)', float),
     ]
 
-    _STACK_IGNORE = ['ethernet.fcs']
+    _STACK_IGNORE = ['ethernet.fcs', 'pfcPause.fcs']
 
     _TYPE_TO_HEADER = {
         'ethernet': 'ethernet',
-        'pfcPause': 'pfc_pause',
+        'pfcPause': 'pfcpause',
         'vlan': 'vlan',
         'ipv4': 'ipv4',
         'ipv6': 'ipv6',
@@ -204,7 +205,7 @@ class TrafficItem(CustomField):
                 self._configure_size(ixn_ce, flow.size)
                 self._configure_rate(ixn_ce, flow.rate)
                 self._configure_tx_control(ixn_ce, hl_stream_count, flow.duration)
-                self._configure_options(flow)
+            self._configure_options()
 
     def _configure_tracking(self, flow, ixn_tracking):
         """Set tracking options"""
@@ -215,12 +216,13 @@ class TrafficItem(CustomField):
         if set(tracking_options) != set(ixn_tracking.TrackBy):
             ixn_tracking.update(TrackBy=tracking_options)
         
-    def _configure_options(self, flow):
+    def _configure_options(self):
         enable_min_frame_size = False
-        if (len(flow.packet) == 1 and flow.packet[0].choice == 'pfcpause'
-                and flow.size is not None
-                and flow.size.choice == 'fixed' and flow.size.fixed <= 64):
-            enable_min_frame_size = True
+        for flow in self._api.config.flows:
+            if (len(flow.packet) == 1 and flow.packet[
+                    0].choice == 'pfcpause'):
+                enable_min_frame_size = True
+                break
         if self._api._traffic.EnableMinFrameSize != enable_min_frame_size:
             self._api._traffic.EnableMinFrameSize = enable_min_frame_size
 
@@ -290,9 +292,8 @@ class TrafficItem(CustomField):
         any stack items so that the stack list matches the headers list.
         If the headers list is empty then use the traffic generator default stack.
         """
-        stacks_to_remove = []
-        ixn_stack = ixn_stream.Stack.find()
         headers = self.adjust_header(headers)
+        ixn_stack = ixn_stream.Stack.find()
         for i in range(0, len(headers)):
             header = headers[i]
             if len(ixn_stack) <= i:
@@ -302,16 +303,34 @@ class TrafficItem(CustomField):
                 if stack_type_id in self._STACK_IGNORE:
                     stack = self._add_stack(ixn_stream, stack, header)
                 elif stack_type_id not in TrafficItem._TYPE_TO_HEADER:
-                    stacks_to_remove.append(ixn_stack[i])
                     stack = self._add_stack(ixn_stream, ixn_stack[i], header)
                 elif TrafficItem._TYPE_TO_HEADER[
                         stack_type_id] != header.choice:
-                    stacks_to_remove.append(ixn_stack[i])
                     stack = self._add_stack(ixn_stream, ixn_stack[i], header)
                 else:
                     stack = ixn_stack[i]
             self._configure_field(stack.Field, header)
-        for stack in stacks_to_remove:
+
+        # scan and compare new stack to overcome IxNetwork stack serialization
+        # then remove additional stack
+        if len(headers) == 0:
+            return
+        stacks_to_remove = []
+        ixn_stack = ixn_stream.Stack.find()
+        header_index = 0
+        for i in range(0, len(ixn_stack)):
+            stack_type_id = ixn_stack[i].StackTypeId
+            if stack_type_id in self._STACK_IGNORE:
+                continue
+            if len(headers) <= header_index:
+                stacks_to_remove.append(ixn_stack[i])
+                continue
+            if TrafficItem._TYPE_TO_HEADER[
+                    stack_type_id] != headers[header_index].choice:
+                stacks_to_remove.append(ixn_stack[i])
+            else:
+                header_index += 1
+        for stack in stacks_to_remove[::-1]:
             stack.Remove()
 
     def _add_stack(self, ixn_stream, ixn_stack, header):
@@ -484,6 +503,18 @@ class TrafficItem(CustomField):
             args['InterBurstGapUnits'] = duration.burst.inter_burst_gap_unit
         self._update(ixn_tx_control, **args)
 
+    def _convert_string_to_regex(self, names):
+        ret_list = []
+        for n in names:
+            ret_list.append(
+                n.replace('(', '\\(').replace(')', '\\)')
+                    .replace('[', '\\[').replace(']', '\\]')
+                    .replace('.', '\\.').replace('*', '\\*')
+                    .replace('+', '\\+').replace('?', '\\?')
+                    .replace('{', '\\{').replace('}', '\\}')
+            )
+        return ret_list
+
     def transmit(self, request):
         """Set flow transmit
         1) If start then start any device protocols that are traffic dependent
@@ -495,22 +526,25 @@ class TrafficItem(CustomField):
         if request and request.flow_names:
             flow_names = request.flow_names
         if len(flow_names) == 1:
-            regex = '^%s$' % flow_names[0]
+            regex = '^%s$' % self._convert_string_to_regex(flow_names)[0]
         elif len(flow_names) > 1:
-            regex = '^(%s)$' % '|'.join(flow_names)
+            regex = '^(%s)$' % '|'.join(
+                self._convert_string_to_regex(flow_names)
+            )
 
         if request.state == 'start':
-            all_flow_names = ' '.join(
-                [flow.name for flow in self._api.config.flows])
-            self._api._traffic_item.find(State='^unapplied$')
+            if len(self._api.config.lags) > 0:
+                self._api._ixnetwork.Lag.find().Start()
             if len(self._api._topology.find()) > 0:
                 with Timer(self._api, 'Devices start'):
                     self._api._ixnetwork.StartAllProtocols('sync')
                     self._api.check_protocol_statistics()
+            if len(self._api._traffic_item.find()) == 0:
+                return
+            self._api._traffic_item.find(State='^unapplied$')
             if len(self._api._traffic_item) > 0:
                 with Timer(self._api, 'Flows generate/apply'):
-                    self._api._traffic_item.Generate()
-                    self._api._traffic.Apply()
+                    self._generate_flows_and_apply()
             self._api._traffic_item.find(State='^started$')
             if len(self._api._traffic_item) == 0:
                 with Timer(self._api, 'Flows clear statistics'):
@@ -518,24 +552,48 @@ class TrafficItem(CustomField):
                         ['waitForPortStatsRefresh', 'waitForTrafficStatsRefresh'])
             self._api._start_capture()
         self._api._traffic_item.find(Name=regex)
-        if request.state == 'start':
-            with Timer(self._api, 'Flows start'):
-                self._api._traffic_item.StartStatelessTrafficBlocking()
-        elif request.state == 'stop':
-            self._api._traffic_item.find(Name=regex, State='^started$')
-            if len(self._api._traffic_item) > 0:
-                with Timer(self._api, 'Flows stop'):
-                    self._api._traffic_item.StopStatelessTrafficBlocking()
-        elif request.state == 'pause':
-            self._api._traffic_item.find(Name=regex, State='^started$')
-            if len(self._api._traffic_item) > 0:
-                with Timer(self._api, 'Flows pause'):
-                    self._api._traffic_item.PauseStatelessTraffic(True)
-        elif request.state == 'resume':
-            self._api._traffic_item.find(Name=regex, State='^stopped$')
-            if len(self._api._traffic_item) > 0:
-                with Timer(self._api, 'Flows resume'):
-                    self._api._traffic_item.PauseStatelessTraffic(False)
+        if len(self._api._traffic_item) > 0:
+            if request.state == 'start':
+                self._api._traffic_item.find(Name=regex, Suspend=True,
+                                             State='^started$')
+                if len(self._api._traffic_item) > 0:
+                    with Timer(self._api, 'Flows resume'):
+                        self._api._traffic_item.PauseStatelessTraffic(False)
+                self._api._traffic_item.find(Name=regex, State='^stopped$')
+                if len(self._api._traffic_item) > 0:
+                    with Timer(self._api, 'Flows start'):
+                        self._api._traffic_item.StartStatelessTrafficBlocking()
+            elif request.state == 'stop':
+                self._api._traffic_item.find(Name=regex, State='^started$')
+                if len(self._api._traffic_item) > 0:
+                    with Timer(self._api, 'Flows stop'):
+                        self._api._traffic_item.StopStatelessTrafficBlocking()
+            elif request.state == 'pause':
+                self._api._traffic_item.find(Name=regex, State='^started$')
+                if len(self._api._traffic_item) > 0:
+                    with Timer(self._api, 'Flows pause'):
+                        self._api._traffic_item.PauseStatelessTraffic(True)
+        if request.state == 'stop':
+            if len(self._api._topology.find()) > 0:
+                with Timer(self._api, 'Devices stop'):
+                    self._api._ixnetwork.StopAllProtocols('sync')
+
+    def _generate_flows_and_apply(self):
+        url = '%s/traffic/trafficItem' % self._api._ixnetwork.href
+        res = self._api._request('GET', url)
+        hrefs = [
+            j['links'][-1]['href']
+            for j in res if j.get('links') is not None and len(j['links']) > 0
+        ]
+        url = '{}/traffic/trafficItem/operations/generate'.format(
+            self._api._ixnetwork.href
+        )
+        payload = {
+            'arg1': hrefs
+        }
+        self._api._request('POST', url=url, payload=payload)
+        self._api._traffic.Apply()
+        return
 
     def _set_result_value(self,
                           row,
@@ -583,9 +641,12 @@ class TrafficItem(CustomField):
                 request.flow_names) > 0:
             flow_names = request.flow_names
         if len(flow_names) == 1:
-            filter['regex'] = '^%s$' % flow_names[0]
+            filter['regex'] = '^%s$' % \
+                self._convert_string_to_regex(flow_names)[0]
         elif len(flow_names) > 1:
-            filter['regex'] = '^(%s)$' % '|'.join(flow_names)
+            filter['regex'] = '^(%s)$' % '|'.join(
+                self._convert_string_to_regex(flow_names)
+            )
 
         # initialize result values
         flow_rows = {}
@@ -617,6 +678,8 @@ class TrafficItem(CustomField):
                 flow_row = flow_rows[row['Traffic Item'] + row['Tx Port'] + row['Rx Port']]
                 if float(row['Tx Frame Rate']) > 0 or int(row['Tx Frames']) == 0:
                     flow_row['transmit'] = 'started'
+                else:
+                    flow_row['transmit'] = 'stopped'
                 for external_name, internal_name, external_type in self._RESULT_COLUMNS:
                     # keep plugging values for next columns even if the
                     # current one raises exception
