@@ -77,6 +77,19 @@ class Vport(object):
         'speed_10_fd_mbps': 'speed100',
         'speed_10_hd_mbps': 'speed100'
     }
+
+    _SPEED_MODE_MAP = {
+        'speed_1_gbps': 'normal',
+        'speed_10_gbps': 'tengig',
+        'speed_25_gbps': 'twentyfivegig',
+        'speed_40_gbps': 'fortygig',
+        'speed_50_gbps': 'fiftygig',
+        'speed_100_gbps':
+            '^(?!.*(twohundredgig|fourhundredgig)).*hundredgig.*$',
+        'speed_200_gbps': 'twohundredgig',
+        'speed_400_gbps': 'fourhundredgig'
+    }
+    
     _ADVERTISE_MAP = {
         'advertise_one_thousand_mbps': 'speed1000',
         'advertise_one_hundred_fd_mbps': 'speed100fd',
@@ -187,17 +200,26 @@ class Vport(object):
                 imports.append(capture)
         for capture_item in self._api.config.captures:
             for port_name in capture_item.port_names:
+                capture_mode = 'captureTriggerMode'
+                if capture_item.overwrite:
+                    capture_mode = 'captureContinuousMode'
+                    reset = {'xpath': vports[port_name]['xpath'] + '/capture/trigger'}
+                    reset['captureTriggerEnable'] = False
+                    self._import(reset)
+                isEnable = False
+                if capture_item.enable is True:
+                    isEnable = True
                 capture = {
                     'xpath': vports[port_name]['xpath'] + '/capture',
-                    'captureMode': 'captureTriggerMode',
-                    'hardwareEnabled': False,
+                    'captureMode': capture_mode,
+                    'hardwareEnabled': isEnable,
                     'softwareEnabled': False
                 }
                 pallette = {'xpath': capture['xpath'] + '/filterPallette'}
                 filter = {'xpath': capture['xpath'] + '/filter'}
                 trigger = {'xpath': capture['xpath'] + '/trigger'}
-                if capture_item.basic is not None:
-                    capture['hardwareEnabled'] = True
+                if capture_item.basic is not None and len(
+                        capture_item.basic) > 0:
                     filter['captureFilterEnable'] = True
                     trigger['captureTriggerEnable'] = True
                     filter['captureFilterEnable'] = True
@@ -265,6 +287,76 @@ class Vport(object):
                             % (HostReadyTimeout, ', '.join(check_addresses)))
                     time.sleep(2)
 
+    def _get_layer1(self, port):
+        if hasattr(self._api.config, 'layer1') is False:
+            return
+        if len(self._api.config.layer1) == 0:
+            return
+        for layer1 in self._api.config.layer1:
+            for port_names in layer1.port_names:
+                if port.name in port_names:
+                    return layer1
+        
+    def _set_aggregation(self, port, imports):
+        """ If the card has multiple resource group to control speed within port
+        set it according to the speed"""
+
+        resource_group = None
+        location = getattr(port, 'location', None)
+        if location is None or len(location) == 0:
+            return
+        
+        (hostname, cardid, portid) = location.split(';')
+        layer1 = self._get_layer1(port)
+        if layer1 is None:
+            return
+        
+        speed_mode_map = Vport._SPEED_MODE_MAP
+        self._api.info("Checking port %s to set Layer1 speed %s" %
+                       (port.name, layer1.speed))
+        card_info = self._api.select_card_aggregation(location)
+        if 'aggregation' in card_info.keys() and len(card_info[
+                         'aggregation']) > 0:
+            for aggregation in card_info['aggregation']:
+                if portid in [res_port.split('/')[-1] for res_port in aggregation[
+                        'resourcePorts']]:
+                    self._reset_resource_mode(card_info, speed_mode_map)
+                    resource_group = aggregation
+                    break
+        
+        aggregation_mode = None
+        if resource_group is not None:
+            if layer1.speed in speed_mode_map:
+                mode = speed_mode_map[layer1.speed]
+                for available_mode in resource_group['availableModes']:
+                    if re.search(mode, available_mode.lower()) is not None:
+                        layer1.__setattr__('speed_taken_care', True)
+                        aggregation_mode = available_mode
+                        break
+            else:
+                self._api.warning("Speed %s not avialable within internal map" %
+                                  layer1.speed)
+        else:
+            self._api.warning("Please check physical port number for port %s" %
+                              port.name)
+
+        if aggregation_mode is not None:
+            if aggregation_mode != resource_group['mode']:
+                imports.append({
+                    'xpath': resource_group['xpath'],
+                    'mode': aggregation_mode
+                })
+    
+                self._api.info('Setting port %s to resource mode %s' %
+                               (port.name, aggregation_mode))
+            else:
+                self._api.info("Port %s already set to resource mode %s" %
+                               (port.name, aggregation_mode))
+        else:
+            self._api.warning("Speed %s not available within RG of port %s" %
+                              (layer1.speed, port.name))
+            
+    
     def _set_location(self):
         location_supported = True
         try:
@@ -272,15 +364,29 @@ class Vport(object):
                 self._api._ixnetwork.href + '/locations')
         except Exception:
             location_supported = False
-
-        locations = []
+        
         self._add_hosts(60)
+        
+        # calling little bit costly operation? Otherwise we can't handle same port config for
+        # multiple run (_set_card_resource_mode reset to card level). Also some ports are
+        # handling multiple speed ('novusFourByTwentyFiveGigNonFanOut' and 'novusFourByTenGigNonFanOut')
+        imports = []
+        with Timer(self._api,
+                   'Aggregation mode speed change'):
+            for port in self._api.config.ports:
+                self._set_aggregation(port, imports)
+            if self._import(imports) is False:
+                self._api.info('Retrying card resource mode change')
+                self._import(imports)
+                
         vports = self._api.select_vports()
+        locations = []
         imports = []
         clear_locations = []
         for port in self._api.config.ports:
             vport = vports[port.name]
             location = getattr(port, 'location', None)
+
             if location_supported is True:
                 if vport['location'] == location and vport[
                         'connectionState'].startswith('connectedLink'):
@@ -289,6 +395,7 @@ class Vport(object):
                 if len(vport['connectedTo']) > 0 and vport[
                         'connectionState'].startswith('connectedLink'):
                     continue
+                    
             self._api.ixn_objects[port.name] = vport['href']
             vport = {'xpath': vports[port.name]['xpath']}
             if location_supported is True:
@@ -393,28 +500,25 @@ class Vport(object):
         
         self._set_auto_negotiation(vport, layer1, imports)
 
+    def _reset_resource_mode(self, card, speed_mode_map):
+        # add check for novus NOVUS10
+        novus10g_modes = ['normal', 'tenGigAggregation']
+        if re.search('novus', card['description'].lower()) and set(
+                novus10g_modes) == set(card['availableModes']):
+            speed_mode_map['speed_10_gbps'] = 'normal'
+    
     def _set_card_resource_mode(self, vport, layer1, imports):
         """If the card has an aggregation mode set it according to the speed
         """
         if vport['connectionState'] not in [
                 'connectedLinkUp', 'connectedLinkDown'
-        ]:
+        ] or hasattr(layer1, 'speed_taken_care'):
             return
-        speed_mode_map = {
-            'speed_1_gbps': 'normal',
-            'speed_10_gbps': 'tengig',
-            'speed_25_gbps': 'twentyfivegig',
-            'speed_40_gbps': 'fortygig',
-            'speed_50_gbps': 'fiftygig',
-            'speed_100_gbps':
-            '^(?!.*(twohundredgig|fourhundredgig)).*hundredgig.*$',
-            'speed_200_gbps': 'twohundredgig',
-            'speed_400_gbps': 'fourhundredgig'
-        }
+
         aggregation_mode = None
-        if layer1.speed in speed_mode_map:
-            mode = speed_mode_map[layer1.speed]
+        if layer1.speed in Vport._SPEED_MODE_MAP:
             card = self._api.select_chassis_card(vport)
+            mode = Vport._SPEED_MODE_MAP[layer1.speed]
             for available_mode in card['availableModes']:
                 if re.search(mode, available_mode.lower()) is not None:
                     aggregation_mode = available_mode
@@ -496,14 +600,26 @@ class Vport(object):
     def _add_l1config_import(self, vport, proposed_import, imports):
         type = vport['type'].replace('Fcoe', '')
         l1config = vport['l1Config'][type]
+        key_to_remove = []
         for key in proposed_import:
             if key == 'xpath':
                 continue
-            if key in l1config and l1config[key] != proposed_import[key]:
-                imports.append(proposed_import)
-                return
+            if key not in l1config or l1config[key] == proposed_import[key]:
+                key_to_remove.append(key)
+        # add this constrain due to handle some specific use case (1G to 10G)
+        if 'speed' in key_to_remove and 'speedAuto' not in key_to_remove:
+            key_to_remove.remove('speed')
+        for key in key_to_remove:
+            proposed_import.pop(key)
+        if len(proposed_import) > 0:
+            imports.append(proposed_import)
 
     def _set_gigabit_auto_negotiation(self, vport, layer1, imports):
+        advertise = []
+        advertise.append(Vport._SPEED_MAP[layer1.speed])
+        auto_field_name  = 'enableAutoNegotiation'
+        if re.search('novustengiglan', vport['type'].lower()) is not None:
+            auto_field_name = 'autoNegotiate'
         proposed_import = {
             'xpath':
             vport['xpath'] + '/l1Config/' + vport['type'].replace('Fcoe', ''),
@@ -511,15 +627,20 @@ class Vport(object):
             layer1.ieee_media_defaults,
             'speed':
             Vport._SPEED_MAP[layer1.speed],
-            'enableAutoNegotiation':
+            '{0}'.format(auto_field_name):
             False if layer1.auto_negotiate is None else layer1.auto_negotiate,
             'enableRsFec':
             False if layer1.auto_negotiation is None else
             layer1.auto_negotiation.rs_fec,
             'linkTraining':
             False if layer1.auto_negotiation is None else
-            layer1.auto_negotiation.link_training
+            layer1.auto_negotiation.link_training,
+            'speedAuto':
+            advertise
         }
+        if layer1.media is not None:
+            proposed_import['media'] = layer1.media
+            
         self._add_l1config_import(vport, proposed_import, imports)
 
     def _get_speed(self, vport, layer1):
